@@ -158,7 +158,7 @@ class WSGIHandler(object):
 
     def handle(self):
         try:
-            while True:
+            while self.socket is not None:
                 self.time_start = time.time()
                 self.time_finish = 0
                 result = self.handle_one_request()
@@ -173,6 +173,12 @@ class WSGIHandler(object):
                 self.log_request()
                 break
         finally:
+            if self.socket is not None:
+                try:
+                    self.socket._sock.close()  # do not rely on garbage collection
+                    self.socket.close()
+                except socket.error:
+                    pass
             self.__dict__.pop('socket', None)
             self.__dict__.pop('rfile', None)
             self.__dict__.pop('wfile', None)
@@ -200,8 +206,9 @@ class WSGIHandler(object):
                 self.log_error('Expected GET method: %r', raw_requestline)
                 return
             self.request_version = "HTTP/0.9"
+            # QQQ I'm pretty sure we can drop support for HTTP/0.9
         else:
-            self.log_error('Invalid GET method: %r', raw_requestline)
+            self.log_error('Invalid HTTP method: %r', raw_requestline)
             return
 
         self.headers = self.MessageClass(self.rfile, 0)
@@ -244,22 +251,23 @@ class WSGIHandler(object):
         except Exception:
             traceback.print_exc()
             message = '%r %r' % (msg, args)
-            sys.exc_clear()
         try:
             message = '%s: %s' % (self.socket, message)
         except Exception:
-            sys.exc_clear()
+            pass
         try:
             sys.stderr.write(message + '\n')
         except Exception:
             traceback.print_exc()
-            sys.exc_clear()
+
+    def read_requestline(self):
+        return self.rfile.readline(MAX_REQUEST_LINE)
 
     def handle_one_request(self):
         if self.rfile.closed:
             return
 
-        raw_requestline = self.rfile.readline(MAX_REQUEST_LINE)
+        raw_requestline = self.read_requestline()
         if not raw_requestline:
             return
 
@@ -303,10 +311,6 @@ class WSGIHandler(object):
         if not self.status:
             raise AssertionError("The application did not call start_response()")
         if not self.headers_sent:
-            if hasattr(self.result, '__len__') and 'Content-Length' not in self.response_headers_list:
-                self.response_headers.append(('Content-Length', str(sum(len(chunk) for chunk in self.result))))
-                self.response_headers_list.append('Content-Length')
-
             if 'Date' not in self.response_headers_list:
                 self.response_headers.append(('Date', format_date_time(time.time())))
                 self.response_headers_list.append('Date')
@@ -314,14 +318,21 @@ class WSGIHandler(object):
             if self.request_version == 'HTTP/1.0' and 'Connection' not in self.response_headers_list:
                 self.response_headers.append(('Connection', 'close'))
                 self.response_headers_list.append('Connection')
-                self.close_connection = 1
+                self.close_connection = True
             elif ('Connection', 'close') in self.response_headers:
-                self.close_connection = 1
+                self.close_connection = True
 
-            if self.request_version != 'HTTP/1.0' and 'Content-Length' not in self.response_headers_list:
-                self.response_use_chunked = True
-                self.response_headers.append(('Transfer-Encoding', 'chunked'))
-                self.response_headers_list.append('Transfer-Encoding')
+            if self.code not in [204, 304]:
+                # the reply will include message-body; make sure we have either Content-Length or chunked
+                if 'Content-Length' not in self.response_headers_list:
+                    if hasattr(self.result, '__len__'):
+                        self.response_headers.append(('Content-Length', str(sum(len(chunk) for chunk in self.result))))
+                        self.response_headers_list.append('Content-Length')
+                    else:
+                        if self.request_version != 'HTTP/1.0':
+                            self.response_use_chunked = True
+                            self.response_headers.append(('Transfer-Encoding', 'chunked'))
+                            self.response_headers_list.append('Transfer-Encoding')
 
             towrite.append('%s %s\r\n' % (self.request_version, self.status))
             for header in self.response_headers:
@@ -349,6 +360,7 @@ class WSGIHandler(object):
             finally:
                 # Avoid dangling circular ref
                 exc_info = None
+        self.code = int(status.split(' ', 1)[0])
         self.status = status
         self.response_headers = [('-'.join([x.capitalize() for x in key.split('-')]), value) for key, value in headers]
         self.response_headers_list = [x[0] for x in self.response_headers]
@@ -361,13 +373,34 @@ class WSGIHandler(object):
 
     def format_request(self):
         now = datetime.now().replace(microsecond=0)
-        return '%s - - [%s] "%s" %s %s %.6f' % (
+        if self.time_finish:
+            delta = '%.6f' % (self.time_finish - self.time_start)
+            length = self.response_length
+        else:
+            delta = '-'
+            if not self.response_length:
+                length = '-'
+        return '%s - - [%s] "%s" %s %s %s' % (
             self.client_address[0],
             now,
             self.requestline,
             (self.status or '000').split()[0],
-            self.response_length,
-            self.time_finish - self.time_start)
+            length,
+            delta)
+
+    def process_result(self):
+        for data in self.result:
+            if data:
+                self.write(data)
+        if self.status and not self.headers_sent:
+            self.write('')
+        if self.response_use_chunked:
+            self.wfile.writelines('0\r\n\r\n')
+            self.response_length += 5
+
+    def run_application(self):
+        self.result = self.application(self.environ, self.start_response)
+        self.process_result()
 
     def handle_one_response(self):
         self.time_start = time.time()
@@ -380,15 +413,7 @@ class WSGIHandler(object):
 
         try:
             try:
-                self.result = self.application(self.environ, self.start_response)
-                for data in self.result:
-                    if data:
-                        self.write(data)
-                if self.status and not self.headers_sent:
-                    self.write('')
-                if self.response_use_chunked:
-                    self.wfile.writelines('0\r\n\r\n')
-                    self.response_length += 5
+                self.run_application()
             except GreenletExit:
                 raise
             except Exception:
@@ -425,9 +450,7 @@ class WSGIHandler(object):
         env['PATH_INFO'] = unquote(path)
         env['QUERY_STRING'] = query
 
-        if self.headers.typeheader is None:
-            env['CONTENT_TYPE'] = self.headers.type
-        else:
+        if self.headers.typeheader is not None:
             env['CONTENT_TYPE'] = self.headers.typeheader
 
         length = self.headers.getheader('content-length')
